'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_NV_present_video'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_NV_present_video',error_checker=_errors._error_checker)
WGL_NUM_VIDEO_SLOTS_NV=_C('WGL_NUM_VIDEO_SLOTS_NV',0x20F0)
@_f
@_p.types(_cs.BOOL,_cs.HDC,_cs.c_uint,_cs.HVIDEOOUTPUTDEVICENV,ctypes.POINTER(_cs.c_int))
def wglBindVideoDeviceNV(hDC,uVideoSlot,hVideoDevice,piAttribList):pass
@_f
@_p.types(_cs.c_int,_cs.HDC,ctypes.POINTER(_cs.HVIDEOOUTPUTDEVICENV))
def wglEnumerateVideoDevicesNV(hDC,phDeviceList):pass
@_f
@_p.types(_cs.BOOL,_cs.c_int,ctypes.POINTER(_cs.c_int))
def wglQueryCurrentContextNV(iAttribute,piValue):pass



'''OpenGL extension NV.multisample_filter_hint

This module customises the behaviour of the 
OpenGL.raw.GL.NV.multisample_filter_hint to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL multisampling typically assumes that the samples of a given
	pixel are weighted uniformly and averaged to compute the pixel's
	resolved color.  This extension provides a hint that permits
	implementations to provide an alternative method of resolving the
	color of multisampled pixels.
	
	As an example of such an alternative method, NVIDIA's GeForce3 GPU
	provides a technique known as Quincunx filtering.  This technique
	is used in two-sample multisampling, but it blends the pixel's two
	samples and three additional samples from adjacent pixels.  The sample
	pattern is analogous to the 5 pattern on a die.  The quality of this
	technique is widely regarded as comparable to 4 sample multisampling.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/multisample_filter_hint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.multisample_filter_hint import *
from OpenGL.raw.GL.NV.multisample_filter_hint import _EXTENSION_NAME

def glInitMultisampleFilterHintNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_light_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_light_texture',error_checker=_errors._error_checker)
GL_ATTENUATION_EXT=_C('GL_ATTENUATION_EXT',0x834D)
GL_FRAGMENT_COLOR_EXT=_C('GL_FRAGMENT_COLOR_EXT',0x834C)
GL_FRAGMENT_DEPTH_EXT=_C('GL_FRAGMENT_DEPTH_EXT',0x8452)
GL_FRAGMENT_MATERIAL_EXT=_C('GL_FRAGMENT_MATERIAL_EXT',0x8349)
GL_FRAGMENT_NORMAL_EXT=_C('GL_FRAGMENT_NORMAL_EXT',0x834A)
GL_SHADOW_ATTENUATION_EXT=_C('GL_SHADOW_ATTENUATION_EXT',0x834E)
GL_TEXTURE_APPLICATION_MODE_EXT=_C('GL_TEXTURE_APPLICATION_MODE_EXT',0x834F)
GL_TEXTURE_LIGHT_EXT=_C('GL_TEXTURE_LIGHT_EXT',0x8350)
GL_TEXTURE_MATERIAL_FACE_EXT=_C('GL_TEXTURE_MATERIAL_FACE_EXT',0x8351)
GL_TEXTURE_MATERIAL_PARAMETER_EXT=_C('GL_TEXTURE_MATERIAL_PARAMETER_EXT',0x8352)
@_f
@_p.types(None,_cs.GLenum)
def glApplyTextureEXT(mode):pass
@_f
@_p.types(None,_cs.GLenum)
def glTextureLightEXT(pname):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glTextureMaterialEXT(face,mode):pass



'''OpenGL extension EXT.shared_texture_palette

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.shared_texture_palette to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_shared_texture_palette defines a shared texture palette which may be
	used in place of the texture object palettes provided by
	EXT_paletted_texture. This is useful for rapidly changing a palette
	common to many textures, rather than having to reload the new palette
	for each texture. The extension acts as a switch, causing all lookups
	that would normally be done on the texture's palette to instead use the
	shared palette.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shared_texture_palette.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.shared_texture_palette import *
from OpenGL.raw.GL.EXT.shared_texture_palette import _EXTENSION_NAME

def glInitSharedTexturePaletteEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.copy_image

This module customises the behaviour of the 
OpenGL.raw.GLX.NV.copy_image to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables efficient image data transfer between image
	objects (i.e. textures and renderbuffers) without the need to bind
	the objects or otherwise configure the rendering pipeline.  The
	WGL and GLX versions allow copying between images in different
	contexts, even if those contexts are in different sharelists or
	even on different physical devices.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/copy_image.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.NV.copy_image import *
from OpenGL.raw.GLX.NV.copy_image import _EXTENSION_NAME

def glInitCopyImageNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.shader_stencil_export

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_stencil_export to provide a more 
Python-friendly API

Overview (from the spec)
	
	In OpenGL, the stencil test is a powerful mechanism to selectively discard
	fragments based on the content of the stencil buffer. However, facilites
	to update the content of the stencil buffer are limited to operations such
	as incrementing the existing value, or overwriting with a fixed reference
	value.
	
	This extension provides a mechanism whereby a shader may generate the
	stencil reference value per invocation. When stencil testing is enabled,
	this allows the test to be performed against the value generated in the
	shader. When the stencil operation is set to GL_REPLACE, this allows a
	value generated in the shader to be written to the stencil buffer directly.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_stencil_export.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_stencil_export import *
from OpenGL.raw.GL.ARB.shader_stencil_export import _EXTENSION_NAME

def glInitShaderStencilExportARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_draw_buffers'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_draw_buffers',error_checker=_errors._error_checker)
GL_DRAW_BUFFER0_ARB=_C('GL_DRAW_BUFFER0_ARB',0x8825)
GL_DRAW_BUFFER10_ARB=_C('GL_DRAW_BUFFER10_ARB',0x882F)
GL_DRAW_BUFFER11_ARB=_C('GL_DRAW_BUFFER11_ARB',0x8830)
GL_DRAW_BUFFER12_ARB=_C('GL_DRAW_BUFFER12_ARB',0x8831)
GL_DRAW_BUFFER13_ARB=_C('GL_DRAW_BUFFER13_ARB',0x8832)
GL_DRAW_BUFFER14_ARB=_C('GL_DRAW_BUFFER14_ARB',0x8833)
GL_DRAW_BUFFER15_ARB=_C('GL_DRAW_BUFFER15_ARB',0x8834)
GL_DRAW_BUFFER1_ARB=_C('GL_DRAW_BUFFER1_ARB',0x8826)
GL_DRAW_BUFFER2_ARB=_C('GL_DRAW_BUFFER2_ARB',0x8827)
GL_DRAW_BUFFER3_ARB=_C('GL_DRAW_BUFFER3_ARB',0x8828)
GL_DRAW_BUFFER4_ARB=_C('GL_DRAW_BUFFER4_ARB',0x8829)
GL_DRAW_BUFFER5_ARB=_C('GL_DRAW_BUFFER5_ARB',0x882A)
GL_DRAW_BUFFER6_ARB=_C('GL_DRAW_BUFFER6_ARB',0x882B)
GL_DRAW_BUFFER7_ARB=_C('GL_DRAW_BUFFER7_ARB',0x882C)
GL_DRAW_BUFFER8_ARB=_C('GL_DRAW_BUFFER8_ARB',0x882D)
GL_DRAW_BUFFER9_ARB=_C('GL_DRAW_BUFFER9_ARB',0x882E)
GL_MAX_DRAW_BUFFERS_ARB=_C('GL_MAX_DRAW_BUFFERS_ARB',0x8824)
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDrawBuffersARB(n,bufs):pass



'''OpenGL extension SGIX.blend_alpha_minmax

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.blend_alpha_minmax to provide a more 
Python-friendly API

Overview (from the spec)
	
	Two additional blending equations are specified using the interface
	defined by EXT_blend_minmax.  These equations are similar to the
	MIN_EXT and MAX_EXT blending equations, but the outcome for all four
	color components is determined by a comparison of just the alpha
	component's source and destination values.  These equations are useful
	in image processing and advanced shading algorithms.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/blend_alpha_minmax.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.blend_alpha_minmax import *
from OpenGL.raw.GL.SGIX.blend_alpha_minmax import _EXTENSION_NAME

def glInitBlendAlphaMinmaxSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.enhanced_layouts

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.enhanced_layouts to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the following functionality to layout qualifiers,
	including broadening the API where this functionality is reflected.
	
	The following are added:
	
	1) Use compile-time constant expressions. E.g.,
	
	      const int start = 6;
	      layout(location = start + 2) int vec4 v;
	
	2) Specify explicit byte offsets within a uniform or shader storage block. 
	   For example, if you want two vec4 variables "batman" and "robin" to
	   appear at byte offsets 0 and 64 in your block, you can say:
	
	      uniform Block {
	        layout(offset = 0) vec4 batman;
	        layout(offset = 64) vec4 robin;
	      };
	
	3) Force alignment within a uniform or shader storage block.  The previous
	   example could also be expressed:
	
	      uniform Block {
	        vec4 batman;
	        layout(align = 64) vec4 robin;
	      };
	
	   This says the member 'robin' must start at the next address that is a 
	   multiple of 64.  It allows constructing the same layout in C and in GLSL
	   without inventing explicit offsets.
	
	   Explicit offsets and aligned offsets can be combined:
	
	      uniform Block {
	        vec4 batman;
	        layout(offset = 44, align = 8) vec4 robin;
	      };
	
	   would make 'robin' be at the first 8-byte aligned address, starting at
	   44, which is 48.  This is more useful when using the *align* at
	   the block level, which will apply to all members.
	
	4) Specify component numbers to more fully utilize the vec4-slot interfaces
	   between shader outputs and shader inputs.
	
	   For example, you could fit the following
	
	      - an array of 32 vec3
	      - a single float
	
	   into the space of 32 vec4 slots using the following code:
	
	      // consume X/Y/Z components of 32 vectors
	      layout(location = 0) in vec3 batman[32];
	
	      // consumes W component of first vector
	      layout(location = 0, component = 3) in float robin;
	
	   Further, an array of vec3 and an array of float can be stored 
	   interleaved, using the following.
	
	      // consumes W component of 32 vectors
	      layout(location = 0, component = 3) in float robin[32];
	
	      // consume X/Y/Z components of 32 vectors
	      layout(location = 0) in vec3 batman[32];
	
	5) Specify transform/feedback buffers, locations, and widths. For example:
	
	       layout(xfb_buffer = 0, xfb_offset = 0)  out vec3 var1;
	       layout(xfb_buffer = 0, xfb_offset = 24) out vec3 var2;
	       layout(xfb_buffer = 1, xfb_offset = 0)  out vec4 var3;
	
	   The second line above says to write var2 out to byte offset 24 of 
	   transform/feedback buffer 0.  (When doing this, output are only 
	   captured when xfb_offset is used.)
	
	   To specify the total number of bytes per entry in a buffer:
	
	       layout(xfb_buffer = 1, xfb_stride = 32) out;
	
	   This is necessary if, say, var3 above, which uses bytes 0-11,
	   does not fully fill the buffer, which in this case takes 32 bytes.
	
	   Use of this feature effectively eliminates the need to use previously 
	   existing API commands to describe the transform feedback layout.
	
	6) Allow locations on input and output blocks for SSO interface matching.
	
	   For example:
	
	      layout(location = 4) in block {
	          vec4 batman;   // gets location 4
	          vec4 robin;    // gets location 5
	          layout(location = 7) vec4 joker;  // gets location 7
	          vec4 riddler;  // location 8
	      };

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/enhanced_layouts.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.enhanced_layouts import *
from OpenGL.raw.GL.ARB.enhanced_layouts import _EXTENSION_NAME

def glInitEnhancedLayoutsARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.texture_compression_vtc

This module customises the behaviour of the 
OpenGL.raw.GL.NV.texture_compression_vtc to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for the VTC 3D texture compression
	formats, which are analogous to the S3TC texture compression formats,
	with the addition of some retiling in the Z direction.  VTC has the
	same compression ratio as S3TC and uses 4x4x1, 4x4x2, (4x4x3 when
	non-power-of-two textures are supported), or 4x4x4 blocks.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_compression_vtc.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.texture_compression_vtc import *
from OpenGL.raw.GL.NV.texture_compression_vtc import _EXTENSION_NAME

def glInitTextureCompressionVtcNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.fence

This module customises the behaviour of the 
OpenGL.raw.GL.NV.fence to provide a more 
Python-friendly API

Overview (from the spec)
	
	The goal of this extension is provide a finer granularity of
	synchronizing GL command completion than offered by standard OpenGL,
	which offers only two mechanisms for synchronization: Flush and Finish.
	Since Flush merely assures the user that the commands complete in a
	finite (though undetermined) amount of time, it is, thus, of only
	modest utility.  Finish, on the other hand, stalls CPU execution
	until all pending GL commands have completed.  This extension offers
	a middle ground - the ability to "finish" a subset of the command
	stream, and the ability to determine whether a given command has
	completed or not.
	
	This extension introduces the concept of a "fence" to the OpenGL
	command stream.  Once the fence is inserted into the command stream, it
	can be queried for a given condition - typically, its completion.
	Moreover, the application may also request a partial Finish -- that is,
	all commands prior to the fence will be forced to complete until control
	is returned to the calling process.  These new mechanisms allow for
	synchronization between the host CPU and the GPU, which may be accessing
	the same resources (typically memory).
	
	This extension is useful in conjunction with NV_vertex_array_range
	to determine when vertex information has been pulled from the
	vertex array range.  Once a fence has been tested TRUE or finished,
	all vertex indices issued before the fence must have been pulled.
	This ensures that the vertex data memory corresponding to the issued
	vertex indices can be safely modified (assuming no other outstanding
	vertex indices are issued subsequent to the fence).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/fence.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.fence import *
from OpenGL.raw.GL.NV.fence import _EXTENSION_NAME

def glInitFenceNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDeleteFencesNV.fences size not checked against n
glDeleteFencesNV=wrapper.wrapper(glDeleteFencesNV).setInputArraySize(
    'fences', None
)
glGenFencesNV=wrapper.wrapper(glGenFencesNV).setOutput(
    'fences',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
glGetFenceivNV=wrapper.wrapper(glGetFenceivNV).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.robustness

This module customises the behaviour of the 
OpenGL.raw.GLES1.EXT.robustness to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/robustness.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.EXT.robustness import *
from OpenGL.raw.GLES1.EXT.robustness import _EXTENSION_NAME

def glInitRobustnessEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glReadnPixelsEXT.data size not checked against bufSize
glReadnPixelsEXT=wrapper.wrapper(glReadnPixelsEXT).setInputArraySize(
    'data', None
)
# INPUT glGetnUniformfvEXT.params size not checked against bufSize
glGetnUniformfvEXT=wrapper.wrapper(glGetnUniformfvEXT).setInputArraySize(
    'params', None
)
# INPUT glGetnUniformivEXT.params size not checked against bufSize
glGetnUniformivEXT=wrapper.wrapper(glGetnUniformivEXT).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIS_pixel_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIS_pixel_texture',error_checker=_errors._error_checker)
GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS=_C('GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS',0x8355)
GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS=_C('GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS',0x8354)
GL_PIXEL_GROUP_COLOR_SGIS=_C('GL_PIXEL_GROUP_COLOR_SGIS',0x8356)
GL_PIXEL_TEXTURE_SGIS=_C('GL_PIXEL_TEXTURE_SGIS',0x8353)
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glGetPixelTexGenParameterfvSGIS(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glGetPixelTexGenParameterivSGIS(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLfloat)
def glPixelTexGenParameterfSGIS(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLfloatArray)
def glPixelTexGenParameterfvSGIS(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glPixelTexGenParameteriSGIS(pname,param):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLintArray)
def glPixelTexGenParameterivSGIS(pname,params):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_INTEL_performance_query'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_INTEL_performance_query',error_checker=_errors._error_checker)
GL_PERFQUERY_COUNTER_DATA_BOOL32_INTEL=_C('GL_PERFQUERY_COUNTER_DATA_BOOL32_INTEL',0x94FC)
GL_PERFQUERY_COUNTER_DATA_DOUBLE_INTEL=_C('GL_PERFQUERY_COUNTER_DATA_DOUBLE_INTEL',0x94FB)
GL_PERFQUERY_COUNTER_DATA_FLOAT_INTEL=_C('GL_PERFQUERY_COUNTER_DATA_FLOAT_INTEL',0x94FA)
GL_PERFQUERY_COUNTER_DATA_UINT32_INTEL=_C('GL_PERFQUERY_COUNTER_DATA_UINT32_INTEL',0x94F8)
GL_PERFQUERY_COUNTER_DATA_UINT64_INTEL=_C('GL_PERFQUERY_COUNTER_DATA_UINT64_INTEL',0x94F9)
GL_PERFQUERY_COUNTER_DESC_LENGTH_MAX_INTEL=_C('GL_PERFQUERY_COUNTER_DESC_LENGTH_MAX_INTEL',0x94FF)
GL_PERFQUERY_COUNTER_DURATION_NORM_INTEL=_C('GL_PERFQUERY_COUNTER_DURATION_NORM_INTEL',0x94F1)
GL_PERFQUERY_COUNTER_DURATION_RAW_INTEL=_C('GL_PERFQUERY_COUNTER_DURATION_RAW_INTEL',0x94F2)
GL_PERFQUERY_COUNTER_EVENT_INTEL=_C('GL_PERFQUERY_COUNTER_EVENT_INTEL',0x94F0)
GL_PERFQUERY_COUNTER_NAME_LENGTH_MAX_INTEL=_C('GL_PERFQUERY_COUNTER_NAME_LENGTH_MAX_INTEL',0x94FE)
GL_PERFQUERY_COUNTER_RAW_INTEL=_C('GL_PERFQUERY_COUNTER_RAW_INTEL',0x94F4)
GL_PERFQUERY_COUNTER_THROUGHPUT_INTEL=_C('GL_PERFQUERY_COUNTER_THROUGHPUT_INTEL',0x94F3)
GL_PERFQUERY_COUNTER_TIMESTAMP_INTEL=_C('GL_PERFQUERY_COUNTER_TIMESTAMP_INTEL',0x94F5)
GL_PERFQUERY_DONOT_FLUSH_INTEL=_C('GL_PERFQUERY_DONOT_FLUSH_INTEL',0x83F9)
GL_PERFQUERY_FLUSH_INTEL=_C('GL_PERFQUERY_FLUSH_INTEL',0x83FA)
GL_PERFQUERY_GLOBAL_CONTEXT_INTEL=_C('GL_PERFQUERY_GLOBAL_CONTEXT_INTEL',0x00000001)
GL_PERFQUERY_GPA_EXTENDED_COUNTERS_INTEL=_C('GL_PERFQUERY_GPA_EXTENDED_COUNTERS_INTEL',0x9500)
GL_PERFQUERY_QUERY_NAME_LENGTH_MAX_INTEL=_C('GL_PERFQUERY_QUERY_NAME_LENGTH_MAX_INTEL',0x94FD)
GL_PERFQUERY_SINGLE_CONTEXT_INTEL=_C('GL_PERFQUERY_SINGLE_CONTEXT_INTEL',0x00000000)
GL_PERFQUERY_WAIT_INTEL=_C('GL_PERFQUERY_WAIT_INTEL',0x83FB)
@_f
@_p.types(None,_cs.GLuint)
def glBeginPerfQueryINTEL(queryHandle):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLuintArray)
def glCreatePerfQueryINTEL(queryId,queryHandle):pass
@_f
@_p.types(None,_cs.GLuint)
def glDeletePerfQueryINTEL(queryHandle):pass
@_f
@_p.types(None,_cs.GLuint)
def glEndPerfQueryINTEL(queryHandle):pass
@_f
@_p.types(None,arrays.GLuintArray)
def glGetFirstPerfQueryIdINTEL(queryId):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLuintArray)
def glGetNextPerfQueryIdINTEL(queryId,nextQueryId):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,arrays.GLcharArray,_cs.GLuint,arrays.GLcharArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuint64Array)
def glGetPerfCounterInfoINTEL(queryId,counterId,counterNameLength,counterName,counterDescLength,counterDesc,counterOffset,counterDataSize,counterTypeEnum,counterDataTypeEnum,rawCounterMaxValue):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,ctypes.c_void_p,arrays.GLuintArray)
def glGetPerfQueryDataINTEL(queryHandle,flags,dataSize,data,bytesWritten):pass
@_f
@_p.types(None,arrays.GLcharArray,arrays.GLuintArray)
def glGetPerfQueryIdByNameINTEL(queryName,queryId):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,arrays.GLcharArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray)
def glGetPerfQueryInfoINTEL(queryId,queryNameLength,queryName,dataSize,noCounters,noInstances,capsMask):pass



import ctypes
from OpenGL.constant import Constant
from OpenGL._bytes import bytes,unicode,as_8_bit, long
from OpenGL._opaque import opaque_pointer_cls as _opaque_pointer_cls
sizeof = ctypes.sizeof

GL_FALSE = Constant( 'GL_FALSE', 0x0 )
GL_TRUE = Constant( 'GL_TRUE', 0x1 )
GL_BYTE = Constant( 'GL_BYTE', 0x1400 )
GL_UNSIGNED_BYTE = Constant( 'GL_UNSIGNED_BYTE', 0x1401 )
GL_SHORT = Constant( 'GL_SHORT', 0x1402 )
GL_UNSIGNED_SHORT = Constant( 'GL_UNSIGNED_SHORT', 0x1403 )
GL_INT = Constant( 'GL_INT', 0x1404 )
GL_UNSIGNED_INT = Constant( 'GL_UNSIGNED_INT', 0x1405 )
GL_UNSIGNED_INT64 = Constant( 'GL_UNSIGNED_INT64_AMD', 0x8BC2 )
GL_FLOAT = Constant( 'GL_FLOAT', 0x1406 )
GL_DOUBLE = Constant( 'GL_DOUBLE', 0x140a )
GL_CHAR = bytes
GL_HALF_NV = Constant( 'GL_HALF_NV', 0x1401 )
GL_VOID_P = object()

BYTE_SIZES = {
    GL_BYTE: 1,
    GL_CHAR: 1,
    GL_UNSIGNED_BYTE: 1,
    GL_SHORT: 2,
    GL_UNSIGNED_SHORT: 2,
    GL_INT: 4,
    GL_UNSIGNED_INT: 4,
    GL_UNSIGNED_INT64: 8,
    GL_FLOAT: 4,
    GL_DOUBLE: 8,
}

ARRAY_TO_GL_TYPE_MAPPING = {
    'c': GL_UNSIGNED_BYTE,
    'f': GL_FLOAT,
    'b': GL_BYTE,
    'i': GL_INT,
    'l': GL_INT,
    '?': GL_INT,# Boolean 
    'd': GL_DOUBLE,
    'L': GL_UNSIGNED_INT,
    'h': GL_SHORT,
    'H': GL_UNSIGNED_SHORT,
    'B': GL_UNSIGNED_BYTE,
    'I': GL_UNSIGNED_INT,
    None: None,
}



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_VERSION_GLX_1_1'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_VERSION_GLX_1_1',error_checker=_errors._error_checker)
GLX_EXTENSIONS=_C('GLX_EXTENSIONS',0x3)
GLX_VENDOR=_C('GLX_VENDOR',0x1)
GLX_VERSION=_C('GLX_VERSION',0x2)
@_f
@_p.types(ctypes.c_char_p,ctypes.POINTER(_cs.Display),_cs.c_int)
def glXGetClientString(dpy,name):pass
@_f
@_p.types(ctypes.c_char_p,ctypes.POINTER(_cs.Display),_cs.c_int)
def glXQueryExtensionsString(dpy,screen):pass
@_f
@_p.types(ctypes.c_char_p,ctypes.POINTER(_cs.Display),_cs.c_int,_cs.c_int)
def glXQueryServerString(dpy,screen,name):pass



'''OpenGL extension EXT.packed_depth_stencil

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.packed_depth_stencil to provide a more 
Python-friendly API

Overview (from the spec)
	
	Many OpenGL implementations have chosen to interleave the depth and
	stencil buffers into one buffer, often with 24 bits of depth
	precision and 8 bits of stencil data.  32 bits is more than is
	needed for the depth buffer much of the time; a 24-bit depth buffer,
	on the other hand, requires that reads and writes of depth data be
	unaligned with respect to power-of-two boundaries.  On the other
	hand, 8 bits of stencil data is more than sufficient for most
	applications, so it is only natural to pack the two buffers into a
	single buffer with both depth and stencil data.  OpenGL never
	provides direct access to the buffers, so the OpenGL implementation
	can provide an interface to applications where it appears the one
	merged buffer is composed of two logical buffers.
	
	One disadvantage of this scheme is that OpenGL lacks any means by
	which this packed data can be handled efficiently.  For example,
	when an application reads from the 24-bit depth buffer, using the
	type GL_UNSIGNED_SHORT will lose 8 bits of data, while
	GL_UNSIGNED_INT has 8 too many.  Both require expensive format
	conversion operations.  A 24-bit format would be no more suitable,
	because it would also suffer from the unaligned memory accesses that
	made the standalone 24-bit depth buffer an unattractive proposition
	in the first place.
	
	Many applications, such as parallel rendering applications, may also
	wish to draw to or read back from both the depth and stencil buffers
	at the same time.  Currently this requires two separate operations,
	reducing performance.  Since the buffers are interleaved, drawing to
	or reading from both should be no more expensive than using just
	one; in some cases, it may even be cheaper.
	
	This extension provides a new data format, GL_DEPTH_STENCIL_EXT,
	that can be used with the glDrawPixels, glReadPixels, and
	glCopyPixels commands, as well as a packed data type,
	GL_UNSIGNED_INT_24_8_EXT, that is meant to be used with
	GL_DEPTH_STENCIL_EXT.  No other data types are supported with
	GL_DEPTH_STENCIL_EXT.  If ARB_depth_texture or SGIX_depth_texture is
	supported, GL_DEPTH_STENCIL_EXT/GL_UNSIGNED_INT_24_8_EXT data can
	also be used for textures; this provides a more efficient way to
	supply data for a 24-bit depth texture.
	
	GL_DEPTH_STENCIL_EXT data, when passed through the pixel path,
	undergoes both depth and stencil operations.  The depth data is
	scaled and biased by the current GL_DEPTH_SCALE and GL_DEPTH_BIAS,
	while the stencil data is shifted and offset by the current
	GL_INDEX_SHIFT and GL_INDEX_OFFSET.  The stencil data is also put
	through the stencil-to-stencil pixel map.
	
	glDrawPixels of GL_DEPTH_STENCIL_EXT data operates similarly to that
	of GL_STENCIL_INDEX data, bypassing the OpenGL fragment pipeline
	entirely, unlike the treatment of GL_DEPTH_COMPONENT data.  The
	stencil and depth masks are applied, as are the pixel ownership and
	scissor tests, but all other operations are skipped.
	
	glReadPixels of GL_DEPTH_STENCIL_EXT data reads back a rectangle
	from both the depth and stencil buffers.
	
	glCopyPixels of GL_DEPTH_STENCIL_EXT data copies a rectangle from
	both the depth and stencil buffers.  Like glDrawPixels, it applies
	both the stencil and depth masks but skips the remainder of the
	OpenGL fragment pipeline.
	
	glTex[Sub]Image[1,2,3]D of GL_DEPTH_STENCIL_EXT data loads depth and
	stencil data into a depth_stencil texture.  glGetTexImage of
	GL_DEPTH_STENCIL_EXT data can be used to retrieve depth and stencil
	data from a depth/stencil texture.
	
	In addition, a new base internal format, GL_DEPTH_STENCIL_EXT, can
	be used by both texture images and renderbuffer storage.  When an
	image with a DEPTH_STENCIL_EXT internal format is attached to both
	the depth and stencil attachment points of a framebuffer object (see
	EXT_framebuffer_object), then it becomes both the depth and stencil
	buffers of the framebuffer.  This fits nicely with hardware that
	interleaves both depth and stencil data into a single buffer.  When
	a texture with DEPTH_STENCIL_EXT data is bound for texturing, only
	the depth component is accessible through the texture fetcher.  The
	stencil data can be written with TexImage or CopyTexImage, and can
	be read with GetTexImage.  When a DEPTH_STENCIL_EXT image is
	attached to the stencil attachment of the bound framebuffer object,
	the stencil data can be accessed through any operation that reads
	from or writes to the framebuffer's stencil buffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/packed_depth_stencil.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.packed_depth_stencil import *
from OpenGL.raw.GL.EXT.packed_depth_stencil import _EXTENSION_NAME

def glInitPackedDepthStencilEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION
# Setup the new image types
from OpenGL import images 
from OpenGL.raw.GL.VERSION.GL_1_1 import GL_UNSIGNED_INT
images.TYPE_TO_ARRAYTYPE[ GL_UNSIGNED_INT_24_8_EXT ] = GL_UNSIGNED_INT
images.TIGHT_PACK_FORMATS[ GL_UNSIGNED_INT_24_8_EXT ] = 4
images.COMPONENT_COUNTS[ GL_DEPTH_STENCIL_EXT ] = 4



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_texture_cube_map'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_texture_cube_map',error_checker=_errors._error_checker)
GL_MAX_CUBE_MAP_TEXTURE_SIZE_OES=_C('GL_MAX_CUBE_MAP_TEXTURE_SIZE_OES',0x851C)
GL_NORMAL_MAP_OES=_C('GL_NORMAL_MAP_OES',0x8511)
GL_REFLECTION_MAP_OES=_C('GL_REFLECTION_MAP_OES',0x8512)
GL_TEXTURE_BINDING_CUBE_MAP_OES=_C('GL_TEXTURE_BINDING_CUBE_MAP_OES',0x8514)
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_OES=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_X_OES',0x8516)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_OES=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_OES',0x8518)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_OES=_C('GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_OES',0x851A)
GL_TEXTURE_CUBE_MAP_OES=_C('GL_TEXTURE_CUBE_MAP_OES',0x8513)
GL_TEXTURE_CUBE_MAP_POSITIVE_X_OES=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_X_OES',0x8515)
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_OES=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_Y_OES',0x8517)
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_OES=_C('GL_TEXTURE_CUBE_MAP_POSITIVE_Z_OES',0x8519)
GL_TEXTURE_GEN_MODE_OES=_C('GL_TEXTURE_GEN_MODE_OES',0x2500)
GL_TEXTURE_GEN_STR_OES=_C('GL_TEXTURE_GEN_STR_OES',0x8D60)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetTexGenfvOES(coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetTexGenivOES(coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfixedArray)
def glGetTexGenxvOES(coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glTexGenfOES(coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glTexGenfvOES(coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glTexGeniOES(coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glTexGenivOES(coord,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLfixed)
def glTexGenxOES(coord,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfixedArray)
def glTexGenxvOES(coord,pname,params):pass



# -*- coding: iso-8859-1 -*-
"""Raw (C-style) API for OpenGL.GLE

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw.GLE.constants import *

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL.raw.GL import _types as GL_types
GLvoid = GL_types.GLvoid




gleDouble = c_double
gleAffine = gleDouble * 3 * 2


# /usr/include/GL/gle.h 136
gleExtrusion = platform.createBaseFunction( 
    'gleExtrusion', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,c_int,arrays.GLdoubleArray,arrays.GLfloatArray],
    doc='gleExtrusion( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), c_int(npoints), arrays.GLdoubleArray(point_array), arrays.GLfloatArray(color_array) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'npoints', 'point_array', 'color_array'),
)


# /usr/include/GL/gle.h 110
gleGetJoinStyle = platform.createBaseFunction( 
    'gleGetJoinStyle', dll=platform.PLATFORM.GLE, resultType=c_int, 
    argTypes=[],
    doc='gleGetJoinStyle(  ) -> c_int', 
    argNames=(),
)


# /usr/include/GL/gle.h 114
gleGetNumSides = platform.createBaseFunction( 
    'gleGetNumSides', dll=platform.PLATFORM.GLE, resultType=c_int, 
    argTypes=[],
    doc='gleGetNumSides(  ) -> c_int', 
    argNames=(),
)


# /usr/include/GL/gle.h 195
gleHelicoid = platform.createBaseFunction( 
    'gleHelicoid', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[gleDouble,gleDouble,gleDouble,gleDouble,gleDouble,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble],
    doc='gleHelicoid( gleDouble(rToroid), gleDouble(startRadius), gleDouble(drdTheta), gleDouble(startZ), gleDouble(dzdTheta), arrays.GLdoubleArray(startXform), arrays.GLdoubleArray(dXformdTheta), gleDouble(startTheta), gleDouble(sweepTheta) ) -> None', 
    argNames=('rToroid', 'startRadius', 'drdTheta', 'startZ', 'dzdTheta', 'startXform', 'dXformdTheta', 'startTheta', 'sweepTheta'),
)


# /usr/include/GL/gle.h 184
gleLathe = platform.createBaseFunction( 
    'gleLathe', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble,gleDouble,gleDouble,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble],
    doc='gleLathe( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), gleDouble(startRadius), gleDouble(drdTheta), gleDouble(startZ), gleDouble(dzdTheta), arrays.GLdoubleArray(startXform), arrays.GLdoubleArray(dXformdTheta), gleDouble(startTheta), gleDouble(sweepTheta) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'startRadius', 'drdTheta', 'startZ', 'dzdTheta', 'startXform', 'dXformdTheta', 'startTheta', 'sweepTheta'),
)


# /usr/include/GL/gle.h 127
glePolyCone = platform.createBaseFunction( 
    'glePolyCone', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLfloatArray,arrays.GLdoubleArray],
    doc='glePolyCone( c_int(npoints), arrays.GLdoubleArray(point_array), arrays.GLfloatArray(color_array), arrays.GLdoubleArray(radius_array) ) -> None', 
    argNames=('npoints', 'point_array', 'color_array', 'radius_array'),
)


# /usr/include/GL/gle.h 121
glePolyCylinder = platform.createBaseFunction( 
    'glePolyCylinder', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLfloatArray,gleDouble],
    doc='glePolyCylinder( c_int(npoints), arrays.GLdoubleArray(point_array), arrays.GLfloatArray(color_array), gleDouble(radius) ) -> None', 
    argNames=('npoints', 'point_array', 'color_array', 'radius'),
)


# /usr/include/GL/gle.h 215
gleScrew = platform.createBaseFunction( 
    'gleScrew', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble,gleDouble],
    doc='gleScrew( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), gleDouble(startz), gleDouble(endz), gleDouble(twist) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'startz', 'endz', 'twist'),
)

# /usr/include/GL/gle.h 111
gleSetJoinStyle = platform.createBaseFunction( 
    'gleSetJoinStyle', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int],
    doc='gleSetJoinStyle( c_int(style) ) -> None', 
    argNames=('style',),
)


# /usr/include/GL/gle.h 115
gleSetNumSides = platform.createBaseFunction( 
    'gleSetNumSides', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int],
    doc='gleSetNumSides( c_int(slices) ) -> None', 
    argNames=('slices',),
)


# /usr/include/GL/gle.h 170
gleSpiral = platform.createBaseFunction( 
    'gleSpiral', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble,gleDouble,gleDouble,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble],
    doc='gleSpiral( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), gleDouble(startRadius), gleDouble(drdTheta), gleDouble(startZ), gleDouble(dzdTheta), arrays.GLdoubleArray(startXform), arrays.GLdoubleArray(dXformdTheta), gleDouble(startTheta), gleDouble(sweepTheta) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'startRadius', 'drdTheta', 'startZ', 'dzdTheta', 'startXform', 'dXformdTheta', 'startTheta', 'sweepTheta'),
)


# /usr/include/GL/gle.h 156
gleSuperExtrusion = platform.createBaseFunction( 
    'gleSuperExtrusion', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,c_int,arrays.GLdoubleArray,arrays.GLfloatArray,arrays.GLdoubleArray],
    doc='gleSuperExtrusion( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), c_int(npoints), arrays.GLdoubleArray(point_array), arrays.GLfloatArray(color_array), arrays.GLdoubleArray(xform_array) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'npoints', 'point_array', 'color_array', 'xform_array'),
)


# /usr/include/GL/gle.h 217
gleTextureMode = platform.createBaseFunction( 
    'gleTextureMode', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int],
    doc='gleTextureMode( c_int(mode) ) -> None', 
    argNames=('mode',),
)


# /usr/include/GL/gle.h 206
gleToroid = platform.createBaseFunction( 
    'gleToroid', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[gleDouble,gleDouble,gleDouble,gleDouble,gleDouble,arrays.GLdoubleArray,arrays.GLdoubleArray,gleDouble,gleDouble],
    doc='gleToroid( gleDouble(rToroid), gleDouble(startRadius), gleDouble(drdTheta), gleDouble(startZ), gleDouble(dzdTheta), arrays.GLdoubleArray(startXform), arrays.GLdoubleArray(dXformdTheta), gleDouble(startTheta), gleDouble(sweepTheta) ) -> None', 
    argNames=('rToroid', 'startRadius', 'drdTheta', 'startZ', 'dzdTheta', 'startXform', 'dXformdTheta', 'startTheta', 'sweepTheta'),
)


# /usr/include/GL/gle.h 146
gleTwistExtrusion = platform.createBaseFunction( 
    'gleTwistExtrusion', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[c_int,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,c_int,arrays.GLdoubleArray,arrays.GLfloatArray,arrays.GLdoubleArray],
    doc='gleTwistExtrusion( c_int(ncp), arrays.GLdoubleArray(contour), arrays.GLdoubleArray(cont_normal), arrays.GLdoubleArray(up), c_int(npoints), arrays.GLdoubleArray(point_array), arrays.GLfloatArray(color_array), arrays.GLdoubleArray(twist_array) ) -> None', 
    argNames=('ncp', 'contour', 'cont_normal', 'up', 'npoints', 'point_array', 'color_array', 'twist_array'),
)


# /usr/include/GL/gle.h 221
rot_about_axis = platform.createBaseFunction( 
    'rot_about_axis', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[gleDouble,arrays.GLdoubleArray],
    doc='rot_about_axis( gleDouble(angle), arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('angle', 'axis'),
)


# /usr/include/GL/gle.h 220
rot_axis = platform.createBaseFunction( 
    'rot_axis', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[gleDouble,arrays.GLdoubleArray],
    doc='rot_axis( gleDouble(omega), arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('omega', 'axis'),
)


# /usr/include/GL/gle.h 222
rot_omega = platform.createBaseFunction( 
    'rot_omega', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray],
    doc='rot_omega( arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('axis',),
)


# /usr/include/GL/gle.h 223
rot_prince = platform.createBaseFunction( 
    'rot_prince', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[gleDouble,c_char],
    doc='rot_prince( gleDouble(omega), c_char(axis) ) -> None', 
    argNames=('omega', 'axis'),
)


# /usr/include/GL/gle.h 225
urot_about_axis = platform.createBaseFunction( 
    'urot_about_axis', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,gleDouble,arrays.GLdoubleArray],
    doc='urot_about_axis( arrays.GLdoubleArray(m), gleDouble(angle), arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('m', 'angle', 'axis'),
)


# /usr/include/GL/gle.h 224
urot_axis = platform.createBaseFunction( 
    'urot_axis', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,gleDouble,arrays.GLdoubleArray],
    doc='urot_axis( arrays.GLdoubleArray(m), gleDouble(omega), arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('m', 'omega', 'axis'),
)


# /usr/include/GL/gle.h 226
urot_omega = platform.createBaseFunction( 
    'urot_omega', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,arrays.GLdoubleArray],
    doc='urot_omega( arrays.GLdoubleArray(m), arrays.GLdoubleArray(axis) ) -> None', 
    argNames=('m', 'axis'),
)


# /usr/include/GL/gle.h 227
urot_prince = platform.createBaseFunction( 
    'urot_prince', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,gleDouble,c_char],
    doc='urot_prince( arrays.GLdoubleArray(m), gleDouble(omega), c_char(axis) ) -> None', 
    argNames=('m', 'omega', 'axis'),
)


# /usr/include/GL/gle.h 232
uview_direction = platform.createBaseFunction( 
    'uview_direction', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray],
    doc='uview_direction( arrays.GLdoubleArray(m), arrays.GLdoubleArray(v21), arrays.GLdoubleArray(up) ) -> None', 
    argNames=('m', 'v21', 'up'),
)


# /usr/include/GL/gle.h 237
uviewpoint = platform.createBaseFunction( 
    'uviewpoint', dll=platform.PLATFORM.GLE, resultType=None, 
    argTypes=[arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray,arrays.GLdoubleArray],
    doc='uviewpoint( arrays.GLdoubleArray(m), arrays.GLdoubleArray(v1), arrays.GLdoubleArray(v2), arrays.GLdoubleArray(up) ) -> None', 
    argNames=('m', 'v1', 'v2', 'up'),
)

__all__ = [
    'GLE_TEXTURE_ENABLE',
    'GLE_TEXTURE_NORMAL_CYL',
    'GLE_TEXTURE_NORMAL_FLAT',
    'GLE_TEXTURE_NORMAL_MODEL_CYL',
    'GLE_TEXTURE_NORMAL_MODEL_FLAT',
    'GLE_TEXTURE_NORMAL_MODEL_SPH',
    'GLE_TEXTURE_NORMAL_SPH',
    'GLE_TEXTURE_STYLE_MASK',
    'GLE_TEXTURE_VERTEX_CYL',
    'GLE_TEXTURE_VERTEX_FLAT',
    'GLE_TEXTURE_VERTEX_MODEL_CYL',
    'GLE_TEXTURE_VERTEX_MODEL_FLAT',
    'GLE_TEXTURE_VERTEX_MODEL_SPH',
    'GLE_TEXTURE_VERTEX_SPH',
    'TUBE_CONTOUR_CLOSED',
    'TUBE_JN_ANGLE',
    'TUBE_JN_CAP',
    'TUBE_JN_CUT',
    'TUBE_JN_MASK',
    'TUBE_JN_RAW',
    'TUBE_JN_ROUND',
    'TUBE_NORM_EDGE',
    'TUBE_NORM_FACET',
    'TUBE_NORM_MASK',
    'TUBE_NORM_PATH_EDGE',
    '__GLE_DOUBLE',
    'gleAffine',
    'gleDouble',
    'gleExtrusion',
    'gleGetJoinStyle',
    'gleGetNumSides',
    'gleHelicoid',
    'gleLathe',
    'glePolyCone',
    'glePolyCylinder',
    'gleScrew',
    'gleSetJoinStyle',
    'gleSetNumSides',
    'gleSpiral',
    'gleSuperExtrusion',
    'gleTextureMode',
    'gleToroid',
    'gleTwistExtrusion',
    'rot_about_axis',
    'rot_axis',
    'rot_omega',
    'rot_prince',
    'urot_about_axis',
    'urot_axis',
    'urot_omega',
    'urot_prince',
    'uview_direction',
    'uviewpoint'
]



'''OpenGL extension ARB.blend_func_extended

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.blend_func_extended to provide a more 
Python-friendly API

Overview (from the spec)
	
	Traditional OpenGL includes fixed-function blending that combines source
	colors with the existing content of a render buffer in a variety of ways.
	A number of extensions have enhanced this functionality by adding further
	sources of blending weights and methods to combine them. However, the inputs
	to the fixed-function blending units are constrained to a source color (as
	output from fragment shading), destination color (as the current content
	of the frame buffer) or constants that may be used in their place.
	
	This extension adds new blending functions whereby a fragment shader may
	output two colors, one of which is treated as the source color, and the
	other used as a blending factor for either source or destination colors.
	Furthermore, this extension increases orthogonality by allowing the
	SRC_ALPHA_SATURATE function to be used as the destination weight.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/blend_func_extended.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.blend_func_extended import *
from OpenGL.raw.GL.ARB.blend_func_extended import _EXTENSION_NAME

def glInitBlendFuncExtendedARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.blend_square

This module customises the behaviour of the 
OpenGL.raw.GL.NV.blend_square to provide a more 
Python-friendly API

Overview (from the spec)
	
	It is useful to be able to multiply a number by itself in the blending
	stages -- for example, in certain types of specular lighting effects
	where a result from a dot product needs to be taken to a high power.
	
	This extension provides four additional blending factors to permit
	this and other effects: SRC_COLOR and ONE_MINUS_SRC_COLOR for source
	blending factors, and DST_COLOR and ONE_MINUS_DST_COLOR for destination
	blending factors.
	
	Direct3D provides capability bits for advertising these additional
	blend modes.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/blend_square.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.blend_square import *
from OpenGL.raw.GL.NV.blend_square import _EXTENSION_NAME

def glInitBlendSquareNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_NV_DX_interop'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_NV_DX_interop',error_checker=_errors._error_checker)
WGL_ACCESS_READ_ONLY_NV=_C('WGL_ACCESS_READ_ONLY_NV',0x00000000)
WGL_ACCESS_READ_WRITE_NV=_C('WGL_ACCESS_READ_WRITE_NV',0x00000001)
WGL_ACCESS_WRITE_DISCARD_NV=_C('WGL_ACCESS_WRITE_DISCARD_NV',0x00000002)
@_f
@_p.types(_cs.BOOL,_cs.HANDLE)
def wglDXCloseDeviceNV(hDevice):pass
@_f
@_p.types(_cs.BOOL,_cs.HANDLE,_cs.GLint,ctypes.POINTER(_cs.HANDLE))
def wglDXLockObjectsNV(hDevice,count,hObjects):pass
@_f
@_p.types(_cs.BOOL,_cs.HANDLE,_cs.GLenum)
def wglDXObjectAccessNV(hObject,access):pass
@_f
@_p.types(_cs.HANDLE,ctypes.c_void_p)
def wglDXOpenDeviceNV(dxDevice):pass
@_f
@_p.types(_cs.HANDLE,_cs.HANDLE,ctypes.c_void_p,_cs.GLuint,_cs.GLenum,_cs.GLenum)
def wglDXRegisterObjectNV(hDevice,dxObject,name,type,access):pass
@_f
@_p.types(_cs.BOOL,ctypes.c_void_p,_cs.HANDLE)
def wglDXSetResourceShareHandleNV(dxObject,shareHandle):pass
@_f
@_p.types(_cs.BOOL,_cs.HANDLE,_cs.GLint,ctypes.POINTER(_cs.HANDLE))
def wglDXUnlockObjectsNV(hDevice,count,hObjects):pass
@_f
@_p.types(_cs.BOOL,_cs.HANDLE,_cs.HANDLE)
def wglDXUnregisterObjectNV(hDevice,hObject):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_ES3_compatibility'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_ES3_compatibility',error_checker=_errors._error_checker)
GL_ANY_SAMPLES_PASSED_CONSERVATIVE=_C('GL_ANY_SAMPLES_PASSED_CONSERVATIVE',0x8D6A)
GL_COMPRESSED_R11_EAC=_C('GL_COMPRESSED_R11_EAC',0x9270)
GL_COMPRESSED_RG11_EAC=_C('GL_COMPRESSED_RG11_EAC',0x9272)
GL_COMPRESSED_RGB8_ETC2=_C('GL_COMPRESSED_RGB8_ETC2',0x9274)
GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2=_C('GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2',0x9276)
GL_COMPRESSED_RGBA8_ETC2_EAC=_C('GL_COMPRESSED_RGBA8_ETC2_EAC',0x9278)
GL_COMPRESSED_SIGNED_R11_EAC=_C('GL_COMPRESSED_SIGNED_R11_EAC',0x9271)
GL_COMPRESSED_SIGNED_RG11_EAC=_C('GL_COMPRESSED_SIGNED_RG11_EAC',0x9273)
GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC=_C('GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC',0x9279)
GL_COMPRESSED_SRGB8_ETC2=_C('GL_COMPRESSED_SRGB8_ETC2',0x9275)
GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2=_C('GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2',0x9277)
GL_MAX_ELEMENT_INDEX=_C('GL_MAX_ELEMENT_INDEX',0x8D6B)
GL_PRIMITIVE_RESTART_FIXED_INDEX=_C('GL_PRIMITIVE_RESTART_FIXED_INDEX',0x8D69)




"""The GLU library implementation via ctypes"""
from OpenGL import platform
from OpenGL.error import *
from OpenGL.raw.GLU import *
from OpenGL.raw.GLU.annotations import *

from OpenGL.GLU.quadrics import *
from OpenGL.GLU.projection import *
from OpenGL.GLU.tess import *
from OpenGL.GLU.glunurbs import *
import ctypes

gluErrorString.restype = ctypes.c_char_p
gluGetString.restype = ctypes.c_char_p


'''OpenGL extension ARB.debug_output

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.debug_output to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows the GL to notify applications when various
	events occur that may be useful during application development and
	debugging.
	
	These events are represented in the form of enumerable messages with
	a human-readable string representation.  Examples of debug events
	include incorrect use of the GL, warnings of undefined behavior, and
	performance warnings.
	
	A message is uniquely identified by a source, a type and an
	implementation-dependent ID within the source and type pair.
	
	A message's source identifies the origin of the message and can
	either describe components of the GL, the window system,
	third-party external sources such as external debuggers, or even
	the application itself.
	
	The type of the message roughly identifies the nature of the event
	that caused the message.  Examples include errors, performance
	warnings, or warnings about undefined behavior.
	
	A message's ID for a given source and type further
	distinguishes messages within those groups.  For example, an error
	caused by a negative parameter value or an invalid internal
	texture format are both errors generated by the API, but would
	likely have different message IDs.
	
	Each message is also assigned to a severity level that denotes
	roughly how "important" that message is in comparison to other
	messages across all sources and types.  For example, notification 
	of a GL error would likely have a higher severity than a performance
	warning due to redundant state changes.
	
	Finally, every message contains an implementation-dependent string
	representation that provides a useful description of the event.
	
	Messages are communicated to the application through an application-
	defined callback function that is called by the GL implementation on
	each debug message.  The motivation for the callback routine is to
	free application developers from actively having to query whether
	a GL error, or any other debuggable event has happened after each
	call to a GL function.  With a callback, developers can keep their
	code free of debug checks, and only have to react to messages as
	they occur.  In situations where using a callback is not possible,
	a message log is also provided that stores copies of recent messages
	until they are actively queried.
	
	To control the volume of debug output, messages can be disabled
	either individually by ID, or entire groups of messages can be
	turned off based on combination of source and type.
	
	The only requirement on the minimum quantity and type of messages
	that implementations of this extension must support is that some
	sort of message must be sent notifying the application whenever any
	GL error occurs.  Any further messages are left to the
	implementation.  Implementations do not have to output messages from
	all sources nor do they have to use all types of messages listed
	by this extension, and both new sources and types can be added by
	other extensions. 
	
	For performance reasons it is recommended, but not required, that
	implementations restrict supporting this extension only to
	contexts created using the debug flag as provided by
	WGL_create_context or GLX_create_context.  This extension places no
	limits on any other functionality provided by debug contexts through
	other extensions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/debug_output.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.debug_output import *
from OpenGL.raw.GL.ARB.debug_output import _EXTENSION_NAME

def glInitDebugOutputARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDebugMessageControlARB.ids size not checked against count
glDebugMessageControlARB=wrapper.wrapper(glDebugMessageControlARB).setInputArraySize(
    'ids', None
)
# INPUT glDebugMessageInsertARB.buf size not checked against length
glDebugMessageInsertARB=wrapper.wrapper(glDebugMessageInsertARB).setInputArraySize(
    'buf', None
)
# INPUT glDebugMessageCallbackARB.userParam size not checked against 'callback'
glDebugMessageCallbackARB=wrapper.wrapper(glDebugMessageCallbackARB).setInputArraySize(
    'userParam', None
)
glGetDebugMessageLogARB=wrapper.wrapper(glGetDebugMessageLogARB).setOutput(
    'ids',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'lengths',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'messageLog',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setOutput(
    'severities',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'sources',size=lambda x:(x,),pnameArg='count',orPassIn=True
).setOutput(
    'types',size=lambda x:(x,),pnameArg='count',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.timer_query

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.timer_query to provide a more 
Python-friendly API

Overview (from the spec)
	
	Applications can benefit from accurate timing information in a number of
	different ways.  During application development, timing information can
	help identify application or driver bottlenecks.  At run time,
	applications can use timing information to dynamically adjust the amount
	of detail in a scene to achieve constant frame rates.  OpenGL
	implementations have historically provided little to no useful timing
	information.  Applications can get some idea of timing by reading timers
	on the CPU, but these timers are not synchronized with the graphics
	rendering pipeline.  Reading a CPU timer does not guarantee the completion
	of a potentially large amount of graphics work accumulated before the
	timer is read, and will thus produce wildly inaccurate results.
	glFinish() can be used to determine when previous rendering commands have
	been completed, but will idle the graphics pipeline and adversely affect
	application performance.
	
	This extension provides a query mechanism that can be used to determine
	the amount of time it takes to fully complete a set of GL commands, and
	without stalling the rendering pipeline.  It uses the query object
	mechanisms first introduced in the occlusion query extension, which allow
	time intervals to be polled asynchronously by the application.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/timer_query.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.timer_query import *
from OpenGL.raw.GL.EXT.timer_query import _EXTENSION_NAME

def glInitTimerQueryEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetQueryObjecti64vEXT=wrapper.wrapper(glGetQueryObjecti64vEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetQueryObjectui64vEXT=wrapper.wrapper(glGetQueryObjectui64vEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


"""The wrapping code for providing natural ctypes-based OpenGL interface"""
import ctypes, logging
from OpenGL import platform, error
from OpenGL._configflags import STORE_POINTERS, ERROR_ON_COPY, SIZE_1_ARRAY_UNPACK
from OpenGL import converters
from OpenGL.converters import DefaultCConverter
from OpenGL.converters import returnCArgument,returnPyArgument
from OpenGL.latebind import LateBind
from OpenGL.arrays import arrayhelpers, arraydatatype
from OpenGL._null import NULL
_log = logging.getLogger( 'OpenGL.wrapper' )

from OpenGL import acceleratesupport
cWrapper = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.latebind import LateBind
        from OpenGL_accelerate.wrapper import (
            Wrapper as cWrapper,
            CArgCalculator,
            PyArgCalculator,
            CArgumentCalculator,
        )
    except ImportError as err:
        _log.warning( """OpenGL_accelerate seems to be installed, but unable to import expected wrapper entry points!""" )

if not STORE_POINTERS:
    if not ERROR_ON_COPY:
        _log.error( """You've specified (not STORE_POINTERS) yet ERROR_ON_COPY is False, this would cause segfaults, so (not STORE_POINTERS) is being ignored""" )
        STORE_POINTERS = True


def asList( o ):
    """Convert to a list if not already one"""
    if not isinstance( o, list ):
        return list(o)
    return o

def none_or_pass( incoming, function, arguments ):
    return incoming
none_or_pass.optional=True

class Wrapper( LateBind ):
    """Wrapper around a ctypes cFunction object providing SWIG-like hooks

    Attributes:

        wrappedOperation -- base operation, normally a ctypes function
            with data-types and error-checking specified
        pyConverters -- converters for incoming Python arguments,
            provide 1:1 mapping to incoming Python arguments, can
            suppress an argument from the argument-set as well
                see setPyConverter
        pyConverterNames -- caching/storage of the argument names
            for the Python converters
        cConverters -- converters for incoming C-level arguments
            produce Python-level objects in 1:1 mapping to ctypes
            arguments from pyConverters results
                see setCConverter
        cResolvers -- converters turning Python-level objects into
            ctypes-compatible data-types
                see setCResolver

    Generic Attributes:

        {ARG1}_LOOKUP_{ARG2} -- lookup dictionaries to provide sizes for
            ARG1 output value from the value of ARG2, provided for
            documentation/reference
        {ARG1}_FROM_{ARG2} -- lookup functions to provide sizes for ARG1
            output value from the value of ARG2, provided for
            documentation/reference
    """
    localProperties = (
        'wrappedOperation',
        '__file__',
        'pyConverters',
        'pyConverterNames',
        'cConverters',
        'cResolvers',
        'storeValues',
        'returnValues',
        '_finalCall',
    )
    def __init__( self, wrappedOperation ):
        """Initialise the wrapper, storing wrappedOperation"""
        if isinstance( wrappedOperation, Wrapper ):
            wrappedOperation = wrappedOperation.wrappedOperation
        self.wrappedOperation = wrappedOperation
    def __getattr__( self, key ):
        """Delegate attribute lookup to our wrappedOperation"""
        if key != 'wrappedOperation':
            return getattr( self.wrappedOperation, key )
        raise AttributeError( key )
    def __nonzero__( self ):
        """Is this function/wrapper available?"""
        return bool( self.wrappedOperation )
    __bool__ = __nonzero__
    def __setattr__( self, key, value ):
        """Forward attribute setting to our wrappedOperation"""
        if key in self.localProperties:
            super( Wrapper, self ).__setattr__( key, value )
        else:
            return setattr( self.wrappedOperation, key, value )
    def pyArgIndex( self, argName ):
        """Return the Python-argument index for the given argument name"""
        argNames = getattr( self, 'pyConverterNames', None )
        if argNames is None:
            argNames = self.wrappedOperation.argNames
        try:
            return asList( argNames ).index( argName )
        except (ValueError,IndexError) as err:
            raise KeyError( """No argument %r in argument list %r"""%(
                argName, argNames
            ))
    def cArgIndex( self, argName ):
        """Return the C-argument index for the given argument name"""
        argNames = self.wrappedOperation.argNames
        try:
            return asList( argNames ).index( argName )
        except (ValueError,IndexError) as err:
            raise KeyError( """No argument %r in argument list %r"""%(
                argName, argNames
            ))
    def setOutput(
        self, outArg, size=(1,), pnameArg=None,
        arrayType=None, oldStyleReturn=SIZE_1_ARRAY_UNPACK,
        orPassIn = False,
    ):
        """Set the given argName to be an output array

        size -- either a tuple compatible with arrayType.zeros or
            a function taking pname to produce such a value.
        arrayType -- array data-type used to generate the output
            array using the zeros class method...
        pnameArg -- optional argument passed into size function, that
            is, the name of the argument whose *value* will be passed
            to the size function, often the name of an input argument
            to be "sized" to match the output argument.
        """
        if arrayType is None:
            # figure out from self.wrappedOperation's argtypes
            index = self.cArgIndex( outArg )
            arrayType = self.wrappedOperation.argtypes[ index ]
            if not hasattr( arrayType, 'asArray' ):
                if arrayType == ctypes.c_void_p:
                    from OpenGL.arrays import GLubyteArray
                    arrayType = GLubyteArray
                else:   
                    raise TypeError( "Should only have array types for output parameters %s on %s is %r"%(
                        outArg, self.wrappedOperation.__name__, arrayType, 
                    ) )
        if pnameArg is None:
            assert not hasattr(size,'__call__' )
            if orPassIn:
                cls = converters.OutputOrInput
            else:
                cls = converters.Output 
            conv = cls(
                name=outArg,
                size=size,
                arrayType=arrayType,
            )
        else:
            if isinstance( size, dict ):
                setattr( self, '%s_LOOKUP_%s'%(outArg,pnameArg), size )
                size = size.__getitem__
            else:
                setattr( self, '%s_FROM_%s'%(outArg,pnameArg), size )
            assert hasattr( size, '__call__' )
            if orPassIn:
                cls = converters.SizedOutputOrInput
            else:
                cls = converters.SizedOutput
            conv = cls(
                name=outArg,
                specifier=pnameArg,
                lookup=size,
                arrayType=arrayType,
            )
        if oldStyleReturn:
            returnObject = conv.oldStyleReturn
        else:
            returnObject = converters.returnCArgument( outArg )
        if orPassIn:
            self.setPyConverter(
                outArg, none_or_pass
            )
        else:
            self.setPyConverter( outArg )
        return self.setCConverter(
            outArg, conv,
        ).setReturnValues(
            returnObject
        )
    def typeOfArg( self, outArg ):
        """Retrieve the defined data-type for the given outArg (name)"""
        index = self.cArgIndex( outArg )
        return self.wrappedOperation.argtypes[ index ]
        
    if not ERROR_ON_COPY:
        def setInputArraySize( self, argName, size=None ):
            """Decorate function with vector-handling code for a single argument
            
            if OpenGL.ERROR_ON_COPY is False, then we return the 
            named argument, converting to the passed array type,
            optionally checking that the array matches size.
            
            if OpenGL.ERROR_ON_COPY is True, then we will dramatically 
            simplify this function, only wrapping if size is True, i.e.
            only wrapping if we intend to do a size check on the array.
            """
            arrayType = self.typeOfArg( argName )
            if not hasattr( arrayType, 'asArray' ):
                if arrayType == ctypes.c_void_p:
                    # special case, we will convert to a void * array...
                    self.setPyConverter( 
                        argName,
                        converters.CallFuncPyConverter( arraydatatype.ArrayDatatype.asArray )
                    )
                    self.setCConverter( argName, converters.getPyArgsName( argName ) )
                    return self
                elif hasattr( arrayType, '_type_' ) and hasattr(arrayType._type_, '_type_' ):
                    # is a ctypes array-of-pointers data-type...
                    # requires special handling no matter what...
                    return self
                else:   
                    raise TypeError( "Should only have array types for output parameters: got %s"%(arrayType,) )
            if size is not None:
                self.setPyConverter( argName, arrayhelpers.asArrayTypeSize(arrayType, size) )
            else:
                self.setPyConverter( argName, arrayhelpers.asArrayType(arrayType) )
            self.setCConverter( argName, converters.getPyArgsName( argName ) )
            return self
    else:
        def setInputArraySize( self, argName, size=None ):
            """Decorate function with vector-handling code for a single argument
            
            if OpenGL.ERROR_ON_COPY is False, then we return the 
            named argument, converting to the passed array type,
            optionally checking that the array matches size.
            
            if OpenGL.ERROR_ON_COPY is True, then we will dramatically 
            simplify this function, only wrapping if size is True, i.e.
            only wrapping if we intend to do a size check on the array.
            """
            if size is not None:
                arrayType = self.typeOfArg( argName )
                # return value is always the source array...
                if hasattr( arrayType, 'asArray' ):
                    self.setPyConverter( argName, arrayhelpers.asArrayTypeSize(arrayType, size) )
                    self.setCConverter( argName, 
                        converters.getPyArgsName( argName ) 
                    )
            return self
    
    def setPyConverter( self, argName, function = NULL ):
        """Set Python-argument converter for given argument

        argName -- the argument name which will be coerced to a usable internal
            format using the function provided.
        function -- None (indicating a simple copy), NULL (default) to eliminate
            the argument from the Python argument-list, or a callable object with
            the signature:

                converter(arg, wrappedOperation, args)

            where arg is the particular argument on which the convert is working,
            wrappedOperation is the underlying wrapper, and args is the set of
            original Python arguments to the function.

        Note that you need exactly the same number of pyConverters as Python
        arguments.
        """
        if not hasattr( self, 'pyConverters' ):
            self.pyConverters = [None]*len( self.wrappedOperation.argNames )
            self.pyConverterNames = list(self.wrappedOperation.argNames)
        try:
            i = asList( self.pyConverterNames ).index( argName )
        except ValueError:
            raise AttributeError( """No argument named %r left in pyConverters for %r: %s"""%(
                argName, self.wrappedOperation.__name__, self.pyConverterNames,
            ))
        if function is NULL:
            del self.pyConverters[i]
            del self.pyConverterNames[i]
        else:
            self.pyConverters[i] = function
        return self
    def setCConverter( self, argName, function ):
        """Set C-argument converter for a given argument

        argName -- the argument name whose C-compatible representation will
            be calculated with the passed function.
        function -- None (indicating a simple copy), a non-callable object to
            be copied into the result-list itself, or a callable object with
            the signature:

                converter( pyArgs, index, wrappedOperation )

            where pyArgs is the set of passed Python arguments, with the
            pyConverters already applied, index is the index of the C argument
            and wrappedOperation is the underlying function.

        C-argument converters are your chance to expand/contract a Python
        argument list (pyArgs) to match the number of arguments expected by
        the ctypes baseOperation.  You can't have a "null" C-argument converter,
        as *something* has to be passed to the C-level function in the
        parameter.
        """
        if not hasattr( self, 'cConverters' ):
            self.cConverters = [None]*len( self.wrappedOperation.argNames )
        try:
            if not isinstance(self.wrappedOperation.argNames, list):
                self.wrappedOperation.argNames = list( self.wrappedOperation.argNames )
            i = asList( self.wrappedOperation.argNames ).index( argName )
        except ValueError:
            raise AttributeError( """No argument named %r left in cConverters: %s"""%(
                argName, self.wrappedOperation.argNames,
            ))
        self.cConverters[i] = function
        return self
    def setCResolver( self, argName, function=NULL ):
        """Set C-argument converter for a given argument"""
        if not hasattr( self, 'cResolvers' ):
            self.cResolvers = [None]*len( self.wrappedOperation.argNames )
        try:
            if not isinstance(self.wrappedOperation.argNames, list):
                self.wrappedOperation.argNames = list( self.wrappedOperation.argNames )
            i = asList( self.wrappedOperation.argNames).index( argName )
        except ValueError:
            raise AttributeError( """No argument named %r left in cConverters: %s"""%(
                argName, self.wrappedOperation.argNames,
            ))
        if function is NULL:
            del self.cResolvers[i]
        else:
            self.cResolvers[i] = function
        return self
    def setStoreValues( self, function=NULL ):
        """Set the storage-of-arguments function for the whole wrapper"""
        if function is NULL or ERROR_ON_COPY and not STORE_POINTERS:
            try:
                del self.storeValues
            except Exception as err:
                pass
        else:
            self.storeValues = function
        return self
    def setReturnValues( self, function=NULL ):
        """Set the return-of-results function for the whole wrapper"""
        if function is NULL:
            try:
                del self.returnValues
            except Exception as err:
                pass
        else:
            self.returnValues = function
        return self
    
    def finalise( self ):
        """Finalise our various elements into simple index-based operations"""
        for attribute in ('pyConverters','cConverters','cResolvers' ):
            value = getattr( self, attribute, None )
            if value is not None:
                for i,item in enumerate(value):
                    if hasattr( item, 'finalise' ):
                        try:
                            item.finalise( self )
                        except Exception as err:
                            raise error.Error(
                                """Error finalising item %s in %s for %s (%r): %s"""%(
                                    i,attribute,self,item,err,
                                )
                            )
        if hasattr( self, 'cConverters' ):
            for i,converter in enumerate( self.cConverters ):
                if isinstance( converter, (type(None),DefaultCConverter )):
                    self.cConverters[i] = DefaultCConverter( self.pyArgIndex( self.argNames[i]) )
        for attribute in ('storeValues','returnValues',):
            item = getattr( self, attribute, None )
            if hasattr( item, 'finalise' ):
                item.finalise( self )
        callFunction = self.finaliseCall()
        if not callFunction:
            raise RuntimeError( """Missing finalised call type for %s"""%( self, ))
        else:
            #self.__class__.finalize = lambda *args: callFunction
            #self.__call__ = callFunction
            #self.__class__.__call__ = callFunction
            #self.__class__.set_call( callFunction )
            #self.__class__.__dict__[ '__call__' ] = callFunction
            #print 'setting class call', callFunction
            self.setFinalCall( callFunction )
            return callFunction
        #return self
    def finaliseCall( self ):
        """Produce specialised versions of call for finalised wrapper object

        This returns a version of __call__ that only does that work which is
        required by the particular wrapper object

        This is essentially a huge set of expanded nested functions, very
        inelegant...
        """
        pyConverters = getattr( self, 'pyConverters', None )
        cConverters = getattr( self, 'cConverters', None )
        cResolvers = getattr( self, 'cResolvers', None )
        wrappedOperation = self.wrappedOperation
        storeValues = getattr( self, 'storeValues', None )
        returnValues = getattr( self, 'returnValues', None )
        if pyConverters:
            if cWrapper:
                calculate_pyArgs = PyArgCalculator(
                    self,pyConverters,
                )
            else:
                pyConverters_mapped = [
                    (i,converter,(converter is None))
                    for (i,converter) in enumerate( pyConverters )
                ]
                pyConverters_length = len([p for p in pyConverters if not getattr( p, 'optional', False)])
                def calculate_pyArgs( args ):
                    if pyConverters_length > len(args):
                        raise ValueError(
                            """%s requires %r arguments (%s), received %s: %r"""%(
                                wrappedOperation.__name__,
                                pyConverters_length,
                                ", ".join( self.pyConverterNames ),
                                len(args),
                                args
                            )
                        )
                    for index,converter,isNone in pyConverters_mapped:
                        if isNone:
                            yield args[index]
                        else:
                            try:
                                yield converter(args[index], self, args)
                            except IndexError as err:
                                yield NULL
                            except Exception as err:
                                if hasattr( err, 'args' ):
                                    err.args += ( converter, )
                                raise
        else:
            calculate_pyArgs = None
        if cConverters:
            if cWrapper:
                calculate_cArgs = CArgCalculator( self, cConverters )
            else:
                cConverters_mapped = [
                    (i,converter,hasattr(converter,'__call__'))
                    for (i,converter) in enumerate( cConverters )
                ]
                def calculate_cArgs( pyArgs ):
                    for index,converter,canCall in cConverters_mapped:
                        if canCall:
                            try:
                                yield converter( pyArgs, index, self )
                            except Exception as err:
                                if hasattr( err, 'args' ):
                                    err.args += (
                                        """Failure in cConverter %r"""%(converter),
                                        pyArgs, index, self,
                                    )
                                raise
                        else:
                            yield converter
        else:
            calculate_cArgs = None
        if cResolvers:
            if cWrapper:
                calculate_cArguments = CArgumentCalculator( cResolvers )
            else:
                cResolvers_mapped = list(enumerate(cResolvers))
                def calculate_cArguments( cArgs ):
                    for i,converter in cResolvers_mapped:
                        if converter is None:
                            yield cArgs[i]
                        else:
                            try:
                                yield converter( cArgs[i] )
                            except Exception as err:
                                err.args += (converter,)
                                raise
        else:
            calculate_cArguments = None
        if cWrapper:
            return cWrapper(
                wrappedOperation,
                calculate_pyArgs=calculate_pyArgs,
                calculate_cArgs=calculate_cArgs,
                calculate_cArguments=calculate_cArguments,
                storeValues=storeValues,
                returnValues=returnValues,
            )
        if pyConverters:
            if cConverters:
                # create a map of index,converter, callable
                if cResolvers:
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
                else:
                    # null cResolvers
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = tuple(calculate_cArgs( pyArgs ))
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
            else:
                # null cConverters
                if cResolvers:
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = pyArgs
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = pyArgs
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = pyArgs
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArgs = pyArgs
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
                else:
                    # null cResolvers
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArguments = pyArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArguments,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArguments,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArguments = pyArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArguments,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArguments = pyArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArguments,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = tuple( calculate_pyArgs( args ))
                                cArguments = pyArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
        else:
            # null pyConverters
            if cConverters:
                if cResolvers:
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
                else:
                    # null cResolvers
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    pyArgs,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                pyArgs = args
                                cArgs = []
                                for (index,converter) in enumerate( cConverters ):
                                    # move enumerate out...
                                    if not hasattr(converter,'__call__'):
                                        cArgs.append( converter )
                                    else:
                                        try:
                                            cArgs.append(
                                                converter( pyArgs, index, self )
                                            )
                                        except Exception as err:
                                            if hasattr( err, 'args' ):
                                                err.args += (
                                                    """Failure in cConverter %r"""%(converter),
                                                    pyArgs, index,
                                                )
                                            raise
                                cArguments = cArgs
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = pyArgs
                                    raise err
                                return result
                            return wrapperCall
            else:
                # null cConverters
                if cResolvers:
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                cArgs = args
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = args
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    args,
                                    cArgs,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    args,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                cArgs = args
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = args
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    args,
                                    cArgs,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                cArgs = args
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = args
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    args,
                                    cArgs,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                cArgs = args
                                cArguments = tuple(calculate_cArguments( cArgs ))
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArgs
                                    err.pyArgs = args
                                    raise err
                                return result
                            return wrapperCall
                else:
                    # null cResolvers
                    if storeValues:
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all possible operations"""
                                cArguments = args
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = args
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    args,
                                    cArguments,
                                )
                                return returnValues(
                                    result,
                                    self,
                                    args,
                                    cArguments,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues"""
                                cArguments = args
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = args
                                    raise err
                                # handle storage of persistent argument values...
                                storeValues(
                                    result,
                                    self,
                                    args,
                                    cArguments,
                                )
                                return result
                            return wrapperCall
                    else: # null storeValues
                        if returnValues:
                            def wrapperCall( *args ):
                                """Wrapper with all save storeValues"""
                                cArguments = args
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = args
                                    raise err
                                return returnValues(
                                    result,
                                    self,
                                    args,
                                    cArguments,
                                )
                            return wrapperCall
                        else:
                            def wrapperCall( *args ):
                                """Wrapper with all save returnValues and storeValues"""
                                cArguments = args
                                try:
                                    result = wrappedOperation( *cArguments )
                                except ctypes.ArgumentError as err:
                                    err.args = err.args + (cArguments,)
                                    raise err
                                except error.GLError as err:
                                    err.cArgs = cArguments
                                    err.pyArgs = args
                                    raise err
                                return result
                            return wrapperCall
#    def __call__( self, *args, **named ):
#        """Finalise the wrapper before calling it"""
#        try:
#            return self._finalCall( *args, **named )
#        except AttributeError, err:
#            return self.finalise()( *args, **named )

    def _unspecialised__call__( self, *args ):
        """Expand arguments, call the function, store values and check errors"""
        pyConverters = getattr( self, 'pyConverters', None )
        if pyConverters:
            if len(pyConverters) != len(args):
                raise ValueError(
                    """%s requires %r arguments (%s), received %s: %r"""%(
                        self.wrappedOperation.__name__,
                        len(pyConverters),
                        ", ".join( self.pyConverterNames ),
                        len(args),
                        args
                    )
                )
            pyArgs = []
            for (converter,arg) in zip(pyConverters,args):
                if converter is None:
                    pyArgs.append( arg )
                else:
                    pyArgs.append( converter(arg, self, args) )
        else:
            pyArgs = args
        cConverters = getattr( self, 'cConverters', None )
        if cConverters:
            cArgs = []
            for (index,converter) in enumerate( cConverters ):
                if not hasattr(converter,'__call__'):
                    cArgs.append( converter )
                else:
                    try:
                        cArgs.append(
                            converter( pyArgs, index, self )
                        )
                    except Exception as err:
                        if hasattr( err, 'args' ):
                            err.args += (
                                """Failure in cConverter %r"""%(converter),
                                pyArgs, index, self,
                            )
                        raise
        else:
            cArgs = pyArgs
        cResolvers = getattr( self, 'cResolvers', None )
        if cResolvers:
            cArguments = []
            for (converter, value) in zip( cResolvers, cArgs ):
                if converter is None:
                    cArguments.append( value )
                else:
                    cArguments.append( converter( value ) )
        else:
            cArguments = cArgs
        try:
            result = self.wrappedOperation( *cArguments )
        except ctypes.ArgumentError as err:
            err.args = err.args + (cArguments,)
            raise err
        except error.GLError as err:
            err.cArgs = cArgs
            err.pyArgs = pyArgs
            raise err
        storeValues = getattr( self, 'storeValues', None )
        if storeValues is not None:
            # handle storage of persistent argument values...
            storeValues(
                result,
                self,
                pyArgs,
                cArgs,
            )
        returnValues = getattr( self, 'returnValues', None )
        if returnValues is not None:
            return returnValues(
                result,
                self,
                pyArgs,
                cArgs,
            )
        else:
            return result

def wrapper( wrappedOperation ):
    """Create a Wrapper sub-class instance for the given wrappedOperation

    The purpose of this function is to create a subclass of Wrapper which
    has the __doc__ and __name__ of the wrappedOperation so that the instance of
    the wrapper will show up as <functionname instance @ address> by default,
    and will have the docstring available naturally in pydoc and the like.
    """
    if isinstance( wrappedOperation, Wrapper ):
        return wrappedOperation
    dict = {
        '__doc__': wrappedOperation.__doc__,
        '__slots__': ('wrappedOperation', ),
    }
    cls = type( wrappedOperation.__name__, (Wrapper,), dict )
    if hasattr( wrappedOperation, '__module__' ):
        cls.__module__ = wrappedOperation.__module__
    instance = cls(wrappedOperation)
    return instance



'''OpenGL extension ARB.draw_instanced

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.draw_instanced to provide a more 
Python-friendly API

Overview (from the spec)
	
	A common use case in GL for some applications is to be able to
	draw the same object, or groups of similar objects that share
	vertex data, primitive count and type, multiple times.  This 
	extension provides a means of accelerating such use cases while 
	restricting the number of API calls, and keeping the amount of 
	duplicate data to a minimum.
	
	This extension introduces two draw calls which are conceptually
	equivalent to a series of draw calls.  Each conceptual call in
	this series is considered an "instance" of the actual draw call.
	
	This extension also introduces a read-only built-in variable to
	GLSL which contains the "instance ID."  This variable initially
	contains 0, but increases by one after each conceptual draw call.
	
	By using the instance ID or multiples thereof as an index into
	a uniform array containing transform data, vertex shaders can 
	draw multiple instances of an object with a single draw call.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/draw_instanced.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.draw_instanced import *
from OpenGL.raw.GL.ARB.draw_instanced import _EXTENSION_NAME

def glInitDrawInstancedARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawElementsInstancedARB.indices size not checked against 'count,type'
glDrawElementsInstancedARB=wrapper.wrapper(glDrawElementsInstancedARB).setInputArraySize(
    'indices', None
)
### END AUTOGENERATED SECTION


"""Platform-specific functions/support for the xorg/X11 windowing system"""
from OpenGL.raw.GLX._types import *
from OpenGL.GLX.VERSION.GLX_1_0 import *
from OpenGL.GLX.VERSION.GLX_1_1 import *
from OpenGL.GLX.VERSION.GLX_1_2 import *
from OpenGL.GLX.VERSION.GLX_1_3 import *
from OpenGL.GLX.VERSION.GLX_1_4 import *



'''OpenGL extension AMD.transform_feedback3_lines_triangles

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.transform_feedback3_lines_triangles to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL 4.0 introduced the ability to record primitives into multiple output
	streams using transform feedback. However, the restriction that all streams
	must output POINT primitives when more than one output stream is active was
	also introduced. This extension simply removes that restriction, allowing
	the same set of primitives to be used with multiple transform feedback
	streams as with a single stream.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/transform_feedback3_lines_triangles.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.transform_feedback3_lines_triangles import *
from OpenGL.raw.GL.AMD.transform_feedback3_lines_triangles import _EXTENSION_NAME

def glInitTransformFeedback3LinesTrianglesAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Load font "constants" (actually void *s) from the GLUT DLL"""
from OpenGL import platform
import logging
_log = logging.getLogger( 'OpenGL.GLUT.fonts' )

for name in [
    'GLUT_STROKE_ROMAN',
    'GLUT_STROKE_MONO_ROMAN',
    'GLUT_BITMAP_9_BY_15',
    'GLUT_BITMAP_8_BY_13',
    'GLUT_BITMAP_TIMES_ROMAN_10',
    'GLUT_BITMAP_TIMES_ROMAN_24',
    'GLUT_BITMAP_HELVETICA_10',
    'GLUT_BITMAP_HELVETICA_12',
    'GLUT_BITMAP_HELVETICA_18',
]:
    try:
        # Win32 just has pointers to values 1,2,3,etc
        # GLX has pointers to font structures...
        p = platform.getGLUTFontPointer( name )
    except (ValueError,AttributeError) as err:
        if platform.PLATFORM.GLUT:
            _log.warning( '''Unable to load font: %s''', name )
        globals()[name] = None
    else:
        globals()[name] = p
        try:
            del p
        except NameError as err:
            pass

try:
    del platform, name
except NameError as err:
    pass



'''OpenGL extension SGIX.fbconfig

This module customises the behaviour of the 
OpenGL.raw.GLX.SGIX.fbconfig to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a new way to describe the capabilities of a 
	GLX drawable (i.e., to describe the depth of color buffer components 
	and the type and size of ancillary buffers), removes the "similarity"
	requirement when making a context current to a drawable, and supports 
	RGBA rendering to one- and two-component Windows and GLX Pixmaps.
	
	Currently GLX overloads X Visuals with information on GLX drawable
	capabilities. This extension defines a new construct, a GLXFBConfigSGIX,
	that encapsulates GLX drawable capabilities and has the following 
	properties:
	
	   - It may or may not have an associated X visual. If it does have 
	   an associated X visual then it is possible to create Windows that  
	   have the capabilities described by the GLXFBConfig.
	
	   - A particular GLXFBConfig does not need to work with all GLX 
	   drawables. For example, it is possible for implementations to export 
	   GLXFBConfigs that only work with GLX pixmaps.
	
	This extension also removes the "similarity" restriction when making
	a context and drawable current. Instead a less restrictive requirement 
	of "compatibility" (see definition below) is imposed. Note that when 
	a context is created it has an associated rendering type which
	is either RGBA or color index. In the future we may want to remove all
	"similarity" and "compatibility" restrictions and allow a context to be
	bound to any drawable that supports its rendering type.
	
	Finally the current GLX specification requires that the GLX_RGBA visual
	attribute be associated only with X visual types TrueColor and DirectColor. 
	This extension defines the semantics for doing RGBA rendering to Windows
	created with visuals of type PseudoColor, StaticColor, GrayScale, and 
	StaticGray.  In each of these cases, the red component is used to
	generate the display, and the green and blue components, if present,
	are ignored for display purposes.  
	
	The OpenGL RGBA rendering semantics are more powerful than the OpenGL
	index rendering semantics.  By extending the number of X visual types
	that can be associated with an RGBA color buffer, this extension allows
	RGBA rendering semantics to be used with pseudo color and gray scale
	displays.  An especially useful application of this extension is
	support of one- and two-component RGBA drawables; drawables whose green,
	blue, and sometimes alpha components have no bitplanes.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/fbconfig.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.SGIX.fbconfig import *
from OpenGL.raw.GLX.SGIX.fbconfig import _EXTENSION_NAME

def glInitFbconfigSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_IBM_static_data'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_IBM_static_data',error_checker=_errors._error_checker)
GL_ALL_STATIC_DATA_IBM=_C('GL_ALL_STATIC_DATA_IBM',103060)
GL_STATIC_VERTEX_ARRAY_IBM=_C('GL_STATIC_VERTEX_ARRAY_IBM',103061)
@_f
@_p.types(None,_cs.GLenum)
def glFlushStaticDataIBM(target):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_gpu_shader_fp64'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_gpu_shader_fp64',error_checker=_errors._error_checker)
GL_DOUBLE=_C('GL_DOUBLE',0x140A)
GL_DOUBLE_MAT2=_C('GL_DOUBLE_MAT2',0x8F46)
GL_DOUBLE_MAT2x3=_C('GL_DOUBLE_MAT2x3',0x8F49)
GL_DOUBLE_MAT2x4=_C('GL_DOUBLE_MAT2x4',0x8F4A)
GL_DOUBLE_MAT3=_C('GL_DOUBLE_MAT3',0x8F47)
GL_DOUBLE_MAT3x2=_C('GL_DOUBLE_MAT3x2',0x8F4B)
GL_DOUBLE_MAT3x4=_C('GL_DOUBLE_MAT3x4',0x8F4C)
GL_DOUBLE_MAT4=_C('GL_DOUBLE_MAT4',0x8F48)
GL_DOUBLE_MAT4x2=_C('GL_DOUBLE_MAT4x2',0x8F4D)
GL_DOUBLE_MAT4x3=_C('GL_DOUBLE_MAT4x3',0x8F4E)
GL_DOUBLE_VEC2=_C('GL_DOUBLE_VEC2',0x8FFC)
GL_DOUBLE_VEC3=_C('GL_DOUBLE_VEC3',0x8FFD)
GL_DOUBLE_VEC4=_C('GL_DOUBLE_VEC4',0x8FFE)
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLdoubleArray)
def glGetUniformdv(program,location,params):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble)
def glUniform1d(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform1dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble)
def glUniform2d(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform2dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glUniform3d(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform3dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glUniform4d(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLdoubleArray)
def glUniform4dv(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2x3dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix2x4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3x2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix3x4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4x2dv(location,count,transpose,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,_cs.GLboolean,arrays.GLdoubleArray)
def glUniformMatrix4x3dv(location,count,transpose,value):pass



'''OpenGL extension APPLE.transform_hint

This module customises the behaviour of the 
OpenGL.raw.GL.APPLE.transform_hint to provide a more 
Python-friendly API

Overview (from the spec)
	
	The transform_hint extension provides a new target,
	TRANSFORM_HINT_APPLE, for the Hint procedure.  When the
	transform hint is set to FASTEST the GL may choose to 
	implement certain state dependent algebraic simplifications
	in the geometry transformation that affect the sub-pixel
	precision of the transformed vertex coordinates.  
	
	For example, if two polygons are rendered with identical object
	coordinates, different GL state settings, and the transform
	hint set to FASTEST, there is no gaurantee that the resulting 
	window coordinates of the two polygons will be precisely
	identical.  Therefore, precise tests of the window coordinates, 
	such as a depth test setting of EQUAL, should not be used.
	
	If the transform hint is set to NICEST or DONT_CARE, two polygons
	with identical object coordinates will always be transformed
	to identical window coordinates.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/transform_hint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.APPLE.transform_hint import *
from OpenGL.raw.GL.APPLE.transform_hint import _EXTENSION_NAME

def glInitTransformHintAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.texture_shader3

This module customises the behaviour of the 
OpenGL.raw.GL.NV.texture_shader3 to provide a more 
Python-friendly API

Overview (from the spec)
	
	NV_texture_shader3 extends the NV_texture_shader functionality by
	adding several new texture shader operations, extending several
	existing texture shader operations, adding a new HILO8 internal
	format, and adding new and more flexible re-mapping modes for dot
	product and dependent texture shader operations.
	
	See the NV_texture_shader extension for information about the
	texture shader operational model.
	
	The fourteen new texture shader operations are:
	
	<offset textures>
	
	24.  OFFSET_PROJECTIVE_TEXTURE_2D_NV - Transforms the signed (ds,dt)
	     components of a previous texture unit by a 2x2 floating-point
	     matrix and then uses the result to offset the stage's texture
	     coordinates for a 2D non-projective texture.
	
	25.  OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV - Same as above except
	     the magnitude component of the previous texture unit result
	     scales the red, green, and blue components of the unsigned RGBA
	     texture 2D access.
	
	26.  OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV - Similar to
	     OFFSET_TEXTURE_2D_NV except that the texture access is into a
	     rectangular non-projective texture.
	
	27.  OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV - Similar to
	     OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV except that the texture
	     access is into a rectangular non-projective texture.
	
	28.  OFFSET_HILO_TEXTURE_2D_NV - Similar to OFFSET_TEXTURE_2D_NV
	     but uses a (higher-precision) HILO base format texture rather
	     than a DSDT-type base format.
	
	29.  OFFSET_HILO_TEXTURE_RECTANGLE_NV - Similar to
	     OFFSET_TEXTURE_RECTANGLE_NV but uses a (higher-precision)
	     HILO base format texture rather than a DSDT-type base format.
	
	30.  OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV - Similar to
	     OFFSET_PROJECTIVE_TEXTURE_2D_NV but uses a (higher-precision)
	     HILO base format texture rather than a DSDT-type base format.
	
	31.  OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV - Similar to
	     OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV but uses a
	     (higher-precision) HILO base format texture rather than a
	     DSDT-type base format.
	
	     (There are no "offset HILO texture scale" operations because
	     HILO textures have only two components with no third component
	     for scaling.)
	
	<dependent textures>
	
	32.  DEPENDENT_HILO_TEXTURE_2D_NV - Converts the hi and lo components
	     of a previous shader HILO result into an (s,t) texture coordinate
	     set to access a 2D non-projective texture.
	
	33.  DEPENDENT_RGB_TEXTURE_3D_NV - Converts the red, green, and
	     blue components of a previous shader RGBA result into an (s,t,r)
	     texture coordinate set to access a 3D non-projective texture.
	
	34.  DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV - Converts the red, green,
	     and blue components of a previous shader RGBA result into an
	     (s,t,r) texture coordinate set to access a cube map texture.
	
	<dot product pass through>
	
	35.  DOT_PRODUCT_PASS_THROUGH_NV - Computes a dot product in the
	     manner of the DOT_PRODUCT_NV operation and the result is [0,1]
	     clamped and smeared to generate the texture unit RGBA result.
	
	<dot product textures>
	
	36.  DOT_PRODUCT_TEXTURE_1D_NV - Computes a dot product in the manner
	     of the DOT_PRODUCT_NV operation and uses the result as the s
	     texture coordinate to access a 2D non-projective texture.
	
	<dot product depth replace>
	
	37.  DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV - Computes a dot product
	     in the manner of the DOT_PRODUCT_NV operation and the result
	     is [0,1] clamped and replaces the fragment's window-space
	     depth value.  The texture unit RGBA result is (0,0,0,0).
	
	Two new internal texture formats have been added: HILO8_NV and
	SIGNED_HILO8_NV.  These texture formats allow HILO textures to be
	stored in half the space; still the filtering for these internal
	texture formats is done with 16-bit precision.
	
	One new unsigned RGBA dot product mapping mode (FORCE_BLUE_TO_ONE_NV)
	forces the blue component to be 1.0 before computing a dot product.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_shader3.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.texture_shader3 import *
from OpenGL.raw.GL.NV.texture_shader3 import _EXTENSION_NAME

def glInitTextureShader3NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NVX_gpu_memory_info'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NVX_gpu_memory_info',error_checker=_errors._error_checker)
GL_GPU_MEMORY_INFO_CURRENT_AVAILABLE_VIDMEM_NVX=_C('GL_GPU_MEMORY_INFO_CURRENT_AVAILABLE_VIDMEM_NVX',0x9049)
GL_GPU_MEMORY_INFO_DEDICATED_VIDMEM_NVX=_C('GL_GPU_MEMORY_INFO_DEDICATED_VIDMEM_NVX',0x9047)
GL_GPU_MEMORY_INFO_EVICTED_MEMORY_NVX=_C('GL_GPU_MEMORY_INFO_EVICTED_MEMORY_NVX',0x904B)
GL_GPU_MEMORY_INFO_EVICTION_COUNT_NVX=_C('GL_GPU_MEMORY_INFO_EVICTION_COUNT_NVX',0x904A)
GL_GPU_MEMORY_INFO_TOTAL_AVAILABLE_MEMORY_NVX=_C('GL_GPU_MEMORY_INFO_TOTAL_AVAILABLE_MEMORY_NVX',0x9048)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_S3_s3tc'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_S3_s3tc',error_checker=_errors._error_checker)
GL_RGB4_S3TC=_C('GL_RGB4_S3TC',0x83A1)
GL_RGBA4_DXT5_S3TC=_C('GL_RGBA4_DXT5_S3TC',0x83A5)
GL_RGBA4_S3TC=_C('GL_RGBA4_S3TC',0x83A3)
GL_RGBA_DXT5_S3TC=_C('GL_RGBA_DXT5_S3TC',0x83A4)
GL_RGBA_S3TC=_C('GL_RGBA_S3TC',0x83A2)
GL_RGB_S3TC=_C('GL_RGB_S3TC',0x83A0)




'''OpenGL extension SGIX.resample

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.resample to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enhances the unpacking resampling capabilities
	of the SGIX_subsample extension.
	
	When pixel data is received from the client and an unpacking
	upsampling mode other than PIXEL_SUBSAMPLE_RATE_4444_SGIX is
	specified, the upsampling is performed via one of two methods:
	RESAMPLE_REPLICATE_SGIX, RESAMPLE_ZERO_FILL_SGIX.
	Replicate and zero fill are provided to
	give the application greatest performance and control over the
	filtering process.
	
	However, when pixel data is read back to the client and a
	packing downsampling mode other than PIXEL_SUBSAMPLE_RATE_4444_SGIX
	is specified, downsampling is
	performed via simple component decimation (point sampling). That is,
	only the RESAMPLE_DECIMATE_SGIX is valid.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/resample.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.resample import *
from OpenGL.raw.GL.SGIX.resample import _EXTENSION_NAME

def glInitResampleSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION